(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
    typeof define === 'function' && define.amd ? define(factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.Cropper = factory());
}(this, (function () { 'use strict';

    function noop() { }
    function run(fn) {
        return fn();
    }
    function blank_object() {
        return Object.create(null);
    }
    function run_all(fns) {
        fns.forEach(run);
    }
    function is_function(thing) {
        return typeof thing === 'function';
    }
    function safe_not_equal(a, b) {
        return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
    }
    function is_empty(obj) {
        return Object.keys(obj).length === 0;
    }

    // Track which nodes are claimed during hydration. Unclaimed nodes can then be removed from the DOM
    // at the end of hydration without touching the remaining nodes.
    let is_hydrating = false;
    function start_hydrating() {
        is_hydrating = true;
    }
    function end_hydrating() {
        is_hydrating = false;
    }
    function upper_bound(low, high, key, value) {
        // Return first index of value larger than input value in the range [low, high)
        while (low < high) {
            const mid = low + ((high - low) >> 1);
            if (key(mid) <= value) {
                low = mid + 1;
            }
            else {
                high = mid;
            }
        }
        return low;
    }
    function init_hydrate(target) {
        if (target.hydrate_init)
            return;
        target.hydrate_init = true;
        // We know that all children have claim_order values since the unclaimed have been detached
        const children = target.childNodes;
        /*
        * Reorder claimed children optimally.
        * We can reorder claimed children optimally by finding the longest subsequence of
        * nodes that are already claimed in order and only moving the rest. The longest
        * subsequence subsequence of nodes that are claimed in order can be found by
        * computing the longest increasing subsequence of .claim_order values.
        *
        * This algorithm is optimal in generating the least amount of reorder operations
        * possible.
        *
        * Proof:
        * We know that, given a set of reordering operations, the nodes that do not move
        * always form an increasing subsequence, since they do not move among each other
        * meaning that they must be already ordered among each other. Thus, the maximal
        * set of nodes that do not move form a longest increasing subsequence.
        */
        // Compute longest increasing subsequence
        // m: subsequence length j => index k of smallest value that ends an increasing subsequence of length j
        const m = new Int32Array(children.length + 1);
        // Predecessor indices + 1
        const p = new Int32Array(children.length);
        m[0] = -1;
        let longest = 0;
        for (let i = 0; i < children.length; i++) {
            const current = children[i].claim_order;
            // Find the largest subsequence length such that it ends in a value less than our current value
            // upper_bound returns first greater value, so we subtract one
            const seqLen = upper_bound(1, longest + 1, idx => children[m[idx]].claim_order, current) - 1;
            p[i] = m[seqLen] + 1;
            const newLen = seqLen + 1;
            // We can guarantee that current is the smallest value. Otherwise, we would have generated a longer sequence.
            m[newLen] = i;
            longest = Math.max(newLen, longest);
        }
        // The longest increasing subsequence of nodes (initially reversed)
        const lis = [];
        // The rest of the nodes, nodes that will be moved
        const toMove = [];
        let last = children.length - 1;
        for (let cur = m[longest] + 1; cur != 0; cur = p[cur - 1]) {
            lis.push(children[cur - 1]);
            for (; last >= cur; last--) {
                toMove.push(children[last]);
            }
            last--;
        }
        for (; last >= 0; last--) {
            toMove.push(children[last]);
        }
        lis.reverse();
        // We sort the nodes being moved to guarantee that their insertion order matches the claim order
        toMove.sort((a, b) => a.claim_order - b.claim_order);
        // Finally, we move the nodes
        for (let i = 0, j = 0; i < toMove.length; i++) {
            while (j < lis.length && toMove[i].claim_order >= lis[j].claim_order) {
                j++;
            }
            const anchor = j < lis.length ? lis[j] : null;
            target.insertBefore(toMove[i], anchor);
        }
    }
    function append(target, node) {
        if (is_hydrating) {
            init_hydrate(target);
            if ((target.actual_end_child === undefined) || ((target.actual_end_child !== null) && (target.actual_end_child.parentElement !== target))) {
                target.actual_end_child = target.firstChild;
            }
            if (node !== target.actual_end_child) {
                target.insertBefore(node, target.actual_end_child);
            }
            else {
                target.actual_end_child = node.nextSibling;
            }
        }
        else if (node.parentNode !== target) {
            target.appendChild(node);
        }
    }
    function insert(target, node, anchor) {
        if (is_hydrating && !anchor) {
            append(target, node);
        }
        else if (node.parentNode !== target || (anchor && node.nextSibling !== anchor)) {
            target.insertBefore(node, anchor || null);
        }
    }
    function detach(node) {
        node.parentNode.removeChild(node);
    }
    function element(name) {
        return document.createElement(name);
    }
    function text(data) {
        return document.createTextNode(data);
    }
    function space() {
        return text(' ');
    }
    function listen(node, event, handler, options) {
        node.addEventListener(event, handler, options);
        return () => node.removeEventListener(event, handler, options);
    }
    function prevent_default(fn) {
        return function (event) {
            event.preventDefault();
            // @ts-ignore
            return fn.call(this, event);
        };
    }
    function attr(node, attribute, value) {
        if (value == null)
            node.removeAttribute(attribute);
        else if (node.getAttribute(attribute) !== value)
            node.setAttribute(attribute, value);
    }
    function children(element) {
        return Array.from(element.childNodes);
    }
    function set_style(node, key, value, important) {
        node.style.setProperty(key, value, important ? 'important' : '');
    }
    function toggle_class(element, name, toggle) {
        element.classList[toggle ? 'add' : 'remove'](name);
    }
    function custom_event(type, detail) {
        const e = document.createEvent('CustomEvent');
        e.initCustomEvent(type, false, false, detail);
        return e;
    }

    let current_component;
    function set_current_component(component) {
        current_component = component;
    }
    function get_current_component() {
        if (!current_component)
            throw new Error('Function called outside component initialization');
        return current_component;
    }
    function onMount(fn) {
        get_current_component().$$.on_mount.push(fn);
    }
    function onDestroy(fn) {
        get_current_component().$$.on_destroy.push(fn);
    }
    function createEventDispatcher() {
        const component = get_current_component();
        return (type, detail) => {
            const callbacks = component.$$.callbacks[type];
            if (callbacks) {
                // TODO are there situations where events could be dispatched
                // in a server (non-DOM) environment?
                const event = custom_event(type, detail);
                callbacks.slice().forEach(fn => {
                    fn.call(component, event);
                });
            }
        };
    }
    // TODO figure out if we still want to support
    // shorthand events, or if we want to implement
    // a real bubbling mechanism
    function bubble(component, event) {
        const callbacks = component.$$.callbacks[event.type];
        if (callbacks) {
            // @ts-ignore
            callbacks.slice().forEach(fn => fn.call(this, event));
        }
    }

    const dirty_components = [];
    const binding_callbacks = [];
    const render_callbacks = [];
    const flush_callbacks = [];
    const resolved_promise = Promise.resolve();
    let update_scheduled = false;
    function schedule_update() {
        if (!update_scheduled) {
            update_scheduled = true;
            resolved_promise.then(flush);
        }
    }
    function add_render_callback(fn) {
        render_callbacks.push(fn);
    }
    let flushing = false;
    const seen_callbacks = new Set();
    function flush() {
        if (flushing)
            return;
        flushing = true;
        do {
            // first, call beforeUpdate functions
            // and update components
            for (let i = 0; i < dirty_components.length; i += 1) {
                const component = dirty_components[i];
                set_current_component(component);
                update(component.$$);
            }
            set_current_component(null);
            dirty_components.length = 0;
            while (binding_callbacks.length)
                binding_callbacks.pop()();
            // then, once components are updated, call
            // afterUpdate functions. This may cause
            // subsequent updates...
            for (let i = 0; i < render_callbacks.length; i += 1) {
                const callback = render_callbacks[i];
                if (!seen_callbacks.has(callback)) {
                    // ...so guard against infinite loops
                    seen_callbacks.add(callback);
                    callback();
                }
            }
            render_callbacks.length = 0;
        } while (dirty_components.length);
        while (flush_callbacks.length) {
            flush_callbacks.pop()();
        }
        update_scheduled = false;
        flushing = false;
        seen_callbacks.clear();
    }
    function update($$) {
        if ($$.fragment !== null) {
            $$.update();
            run_all($$.before_update);
            const dirty = $$.dirty;
            $$.dirty = [-1];
            $$.fragment && $$.fragment.p($$.ctx, dirty);
            $$.after_update.forEach(add_render_callback);
        }
    }
    const outroing = new Set();
    function transition_in(block, local) {
        if (block && block.i) {
            outroing.delete(block);
            block.i(local);
        }
    }

    const globals = (typeof window !== 'undefined'
        ? window
        : typeof globalThis !== 'undefined'
            ? globalThis
            : global);
    function mount_component(component, target, anchor, customElement) {
        const { fragment, on_mount, on_destroy, after_update } = component.$$;
        fragment && fragment.m(target, anchor);
        if (!customElement) {
            // onMount happens before the initial afterUpdate
            add_render_callback(() => {
                const new_on_destroy = on_mount.map(run).filter(is_function);
                if (on_destroy) {
                    on_destroy.push(...new_on_destroy);
                }
                else {
                    // Edge case - component was destroyed immediately,
                    // most likely as a result of a binding initialising
                    run_all(new_on_destroy);
                }
                component.$$.on_mount = [];
            });
        }
        after_update.forEach(add_render_callback);
    }
    function destroy_component(component, detaching) {
        const $$ = component.$$;
        if ($$.fragment !== null) {
            run_all($$.on_destroy);
            $$.fragment && $$.fragment.d(detaching);
            // TODO null out other refs, including component.$$ (but need to
            // preserve final state?)
            $$.on_destroy = $$.fragment = null;
            $$.ctx = [];
        }
    }
    function make_dirty(component, i) {
        if (component.$$.dirty[0] === -1) {
            dirty_components.push(component);
            schedule_update();
            component.$$.dirty.fill(0);
        }
        component.$$.dirty[(i / 31) | 0] |= (1 << (i % 31));
    }
    function init(component, options, instance, create_fragment, not_equal, props, dirty = [-1]) {
        const parent_component = current_component;
        set_current_component(component);
        const $$ = component.$$ = {
            fragment: null,
            ctx: null,
            // state
            props,
            update: noop,
            not_equal,
            bound: blank_object(),
            // lifecycle
            on_mount: [],
            on_destroy: [],
            on_disconnect: [],
            before_update: [],
            after_update: [],
            context: new Map(parent_component ? parent_component.$$.context : options.context || []),
            // everything else
            callbacks: blank_object(),
            dirty,
            skip_bound: false
        };
        let ready = false;
        $$.ctx = instance
            ? instance(component, options.props || {}, (i, ret, ...rest) => {
                const value = rest.length ? rest[0] : ret;
                if ($$.ctx && not_equal($$.ctx[i], $$.ctx[i] = value)) {
                    if (!$$.skip_bound && $$.bound[i])
                        $$.bound[i](value);
                    if (ready)
                        make_dirty(component, i);
                }
                return ret;
            })
            : [];
        $$.update();
        ready = true;
        run_all($$.before_update);
        // `false` as a special case of no DOM component
        $$.fragment = create_fragment ? create_fragment($$.ctx) : false;
        if (options.target) {
            if (options.hydrate) {
                start_hydrating();
                const nodes = children(options.target);
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.l(nodes);
                nodes.forEach(detach);
            }
            else {
                // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                $$.fragment && $$.fragment.c();
            }
            if (options.intro)
                transition_in(component.$$.fragment);
            mount_component(component, options.target, options.anchor, options.customElement);
            end_hydrating();
            flush();
        }
        set_current_component(parent_component);
    }
    /**
     * Base class for Svelte components. Used when dev=false.
     */
    class SvelteComponent {
        $destroy() {
            destroy_component(this, 1);
            this.$destroy = noop;
        }
        $on(type, callback) {
            const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
            callbacks.push(callback);
            return () => {
                const index = callbacks.indexOf(callback);
                if (index !== -1)
                    callbacks.splice(index, 1);
            };
        }
        $set($$props) {
            if (this.$$set && !is_empty($$props)) {
                this.$$.skip_bound = true;
                this.$$set($$props);
                this.$$.skip_bound = false;
            }
        }
    }

    /**
     * Compute the dimension of the crop area based on image size and aspect ratio
     * @param {number} imgWidth width of the src image in pixels
     * @param {number} imgHeight height of the src image in pixels
     * @param {number} aspect aspect ratio of the crop
     */
    function getCropSize(imgWidth, imgHeight, aspect) {
      if (imgWidth >= imgHeight * aspect) {
        return {
          width: imgHeight * aspect,
          height: imgHeight,
        }
      }
      return {
        width: imgWidth,
        height: imgWidth / aspect,
      }
    }

    /**
     * Ensure a new image position stays in the crop area.
     * @param {{x: number, y: number}} position new x/y position requested for the image
     * @param {{width: number, height: number}} imageSize width/height of the src image
     * @param {{width: number, height: number}} cropSize width/height of the crop area
     * @param {number} zoom zoom value
     * @returns {{x: number, y: number}}
     */
    function restrictPosition(position, imageSize, cropSize, zoom) {
      return {
        x: restrictPositionCoord(position.x, imageSize.width, cropSize.width, zoom),
        y: restrictPositionCoord(position.y, imageSize.height, cropSize.height, zoom),
      }
    }

    function restrictPositionCoord(position, imageSize, cropSize, zoom) {
      const maxPosition = (imageSize * zoom) / 2 - cropSize / 2;
      return Math.min(maxPosition, Math.max(position, -maxPosition))
    }

    function getDistanceBetweenPoints(pointA, pointB) {
      return Math.sqrt(Math.pow(pointA.y - pointB.y, 2) + Math.pow(pointA.x - pointB.x, 2))
    }

    /**
     * Compute the output cropped area of the image in percentages and pixels.
     * x/y are the top-left coordinates on the src image
     * @param {{x: number, y number}} crop x/y position of the current center of the image
     * @param {{width: number, height: number, naturalWidth: number, naturelHeight: number}} imageSize width/height of the src image (default is size on the screen, natural is the original size)
     * @param {{width: number, height: number}} cropSize width/height of the crop area
     * @param {number} aspect aspect value
     * @param {number} zoom zoom value
     * @param {boolean} restrictPosition whether we should limit or not the cropped area
     */
    function computeCroppedArea(crop, imgSize, cropSize, aspect, zoom, restrictPosition = true) {
      const limitAreaFn = restrictPosition ? limitArea : noOp;
      const croppedAreaPercentages = {
        x: limitAreaFn(
          100,
          (((imgSize.width - cropSize.width / zoom) / 2 - crop.x / zoom) / imgSize.width) * 100
        ),
        y: limitAreaFn(
          100,
          (((imgSize.height - cropSize.height / zoom) / 2 - crop.y / zoom) / imgSize.height) * 100
        ),
        width: limitAreaFn(100, ((cropSize.width / imgSize.width) * 100) / zoom),
        height: limitAreaFn(100, ((cropSize.height / imgSize.height) * 100) / zoom),
      };

      // we compute the pixels size naively
      const widthInPixels = limitAreaFn(
        imgSize.naturalWidth,
        (croppedAreaPercentages.width * imgSize.naturalWidth) / 100,
        true
      );
      const heightInPixels = limitAreaFn(
        imgSize.naturalHeight,
        (croppedAreaPercentages.height * imgSize.naturalHeight) / 100,
        true
      );
      const isImgWiderThanHigh = imgSize.naturalWidth >= imgSize.naturalHeight * aspect;

      // then we ensure the width and height exactly match the aspect (to avoid rounding approximations)
      // if the image is wider than high, when zoom is 0, the crop height will be equals to iamge height
      // thus we want to compute the width from the height and aspect for accuracy.
      // Otherwise, we compute the height from width and aspect.
      const sizePixels = isImgWiderThanHigh
        ? {
            width: Math.round(heightInPixels * aspect),
            height: heightInPixels,
          }
        : {
            width: widthInPixels,
            height: Math.round(widthInPixels / aspect),
          };
      const croppedAreaPixels = {
        ...sizePixels,
        x: limitAreaFn(
          imgSize.naturalWidth - sizePixels.width,
          (croppedAreaPercentages.x * imgSize.naturalWidth) / 100,
          true
        ),
        y: limitAreaFn(
          imgSize.naturalHeight - sizePixels.height,
          (croppedAreaPercentages.y * imgSize.naturalHeight) / 100,
          true
        ),
      };
      return { croppedAreaPercentages, croppedAreaPixels }
    }

    /**
     * Ensure the returned value is between 0 and max
     * @param {number} max
     * @param {number} value
     * @param {boolean} shouldRound
     */
    function limitArea(max, value, shouldRound = false) {
      const v = shouldRound ? Math.round(value) : value;
      return Math.min(max, Math.max(0, v))
    }

    function noOp(max, value) {
      return value
    }

    /**
     * Return the point that is the center of point a and b
     * @param {{x: number, y: number}} a
     * @param {{x: number, y: number}} b
     */
    function getCenter(a, b) {
      return {
        x: (b.x + a.x) / 2,
        y: (b.y + a.y) / 2,
      }
    }

    /* src/index.svelte generated by Svelte v3.38.3 */

    const { window: window_1 } = globals;

    function add_css() {
    	var style = element("style");
    	style.id = "svelte-12kodkg-style";
    	style.textContent = ".container.svelte-12kodkg{position:absolute;top:0;left:0;right:0;bottom:0;overflow:hidden;user-select:none;touch-action:none;cursor:move}.image.svelte-12kodkg{max-width:100%;max-height:100%;margin:auto;position:absolute;top:0;bottom:0;left:0;right:0;will-change:transform}.cropperArea.svelte-12kodkg{position:absolute;left:50%;top:50%;transform:translate(-50%, -50%);box-shadow:0 0 0 9999em;box-sizing:border-box;color:rgba(0, 0, 0, 0.5);border:1px solid rgba(255, 255, 255, 0.5);overflow:hidden}.grid.svelte-12kodkg:before{content:' ';box-sizing:border-box;border:1px solid rgba(255, 255, 255, 0.5);position:absolute;top:0;bottom:0;left:33.33%;right:33.33%;border-top:0;border-bottom:0}.grid.svelte-12kodkg:after{content:' ';box-sizing:border-box;border:1px solid rgba(255, 255, 255, 0.5);position:absolute;top:33.33%;bottom:33.33%;left:0;right:0;border-left:0;border-right:0}.round.svelte-12kodkg{border-radius:50%}";
    	append(document.head, style);
    }

    // (250:2) {#if cropperSize}
    function create_if_block(ctx) {
    	let div;

    	return {
    		c() {
    			div = element("div");
    			attr(div, "class", "cropperArea svelte-12kodkg");
    			set_style(div, "width", /*cropperSize*/ ctx[7].width + "px");
    			set_style(div, "height", /*cropperSize*/ ctx[7].height + "px");
    			attr(div, "data-testid", "cropper");
    			toggle_class(div, "round", /*cropShape*/ ctx[3] === "round");
    			toggle_class(div, "grid", /*showGrid*/ ctx[4]);
    		},
    		m(target, anchor) {
    			insert(target, div, anchor);
    		},
    		p(ctx, dirty) {
    			if (dirty[0] & /*cropperSize*/ 128) {
    				set_style(div, "width", /*cropperSize*/ ctx[7].width + "px");
    			}

    			if (dirty[0] & /*cropperSize*/ 128) {
    				set_style(div, "height", /*cropperSize*/ ctx[7].height + "px");
    			}

    			if (dirty[0] & /*cropShape*/ 8) {
    				toggle_class(div, "round", /*cropShape*/ ctx[3] === "round");
    			}

    			if (dirty[0] & /*showGrid*/ 16) {
    				toggle_class(div, "grid", /*showGrid*/ ctx[4]);
    			}
    		},
    		d(detaching) {
    			if (detaching) detach(div);
    		}
    	};
    }

    function create_fragment(ctx) {
    	let div;
    	let img;
    	let img_src_value;
    	let t;
    	let mounted;
    	let dispose;
    	let if_block = /*cropperSize*/ ctx[7] && create_if_block(ctx);

    	return {
    		c() {
    			div = element("div");
    			img = element("img");
    			t = space();
    			if (if_block) if_block.c();
    			attr(img, "class", "image svelte-12kodkg");
    			if (img.src !== (img_src_value = /*image*/ ctx[2])) attr(img, "src", img_src_value);
    			attr(img, "alt", "");
    			set_style(img, "transform", "translate(" + /*crop*/ ctx[1].x + "px, " + /*crop*/ ctx[1].y + "px) scale(" + /*zoom*/ ctx[0] + ")");
    			attr(img, "crossorigin", /*crossOrigin*/ ctx[5]);
    			attr(div, "class", "container svelte-12kodkg");
    			attr(div, "data-testid", "container");
    		},
    		m(target, anchor) {
    			insert(target, div, anchor);
    			append(div, img);
    			/*img_binding*/ ctx[25](img);
    			append(div, t);
    			if (if_block) if_block.m(div, null);
    			/*div_binding*/ ctx[26](div);

    			if (!mounted) {
    				dispose = [
    					listen(window_1, "resize", /*computeSizes*/ ctx[10]),
    					listen(window_1, "mousemove", /*onMouseMove*/ ctx[12]),
    					listen(window_1, "mouseup", /*onDragStopped*/ ctx[15], true),
    					listen(window_1, "touchmove", prevent_default(/*onTouchMove*/ ctx[14]), { passive: false }),
    					listen(window_1, "touchend", /*onDragStopped*/ ctx[15]),
    					listen(img, "load", /*onImgLoad*/ ctx[9]),
    					listen(div, "mousedown", prevent_default(/*onMouseDown*/ ctx[11])),
    					listen(div, "touchstart", prevent_default(/*onTouchStart*/ ctx[13])),
    					listen(div, "wheel", prevent_default(/*onWheel*/ ctx[16])),
    					listen(div, "gesturestart", prevent_default(/*gesturestart_handler*/ ctx[23])),
    					listen(div, "gesturechange", prevent_default(/*gesturechange_handler*/ ctx[24]))
    				];

    				mounted = true;
    			}
    		},
    		p(ctx, dirty) {
    			if (dirty[0] & /*image*/ 4 && img.src !== (img_src_value = /*image*/ ctx[2])) {
    				attr(img, "src", img_src_value);
    			}

    			if (dirty[0] & /*crop, zoom*/ 3) {
    				set_style(img, "transform", "translate(" + /*crop*/ ctx[1].x + "px, " + /*crop*/ ctx[1].y + "px) scale(" + /*zoom*/ ctx[0] + ")");
    			}

    			if (dirty[0] & /*crossOrigin*/ 32) {
    				attr(img, "crossorigin", /*crossOrigin*/ ctx[5]);
    			}

    			if (/*cropperSize*/ ctx[7]) {
    				if (if_block) {
    					if_block.p(ctx, dirty);
    				} else {
    					if_block = create_if_block(ctx);
    					if_block.c();
    					if_block.m(div, null);
    				}
    			} else if (if_block) {
    				if_block.d(1);
    				if_block = null;
    			}
    		},
    		i: noop,
    		o: noop,
    		d(detaching) {
    			if (detaching) detach(div);
    			/*img_binding*/ ctx[25](null);
    			if (if_block) if_block.d();
    			/*div_binding*/ ctx[26](null);
    			mounted = false;
    			run_all(dispose);
    		}
    	};
    }

    function instance($$self, $$props, $$invalidate) {
    	let { image } = $$props;
    	let { crop = { x: 0, y: 0 } } = $$props;
    	let { zoom = 1 } = $$props;
    	let { aspect = 4 / 3 } = $$props;
    	let { minZoom = 1 } = $$props;
    	let { maxZoom = 3 } = $$props;
    	let { cropSize = null } = $$props;
    	let { cropShape = "rect" } = $$props;
    	let { showGrid = true } = $$props;
    	let { zoomSpeed = 1 } = $$props;
    	let { crossOrigin = null } = $$props;
    	let { restrictPosition: restrictPosition$1 = true } = $$props;
    	let cropperSize = null;

    	let imageSize = {
    		width: 0,
    		height: 0,
    		naturalWidth: 0,
    		naturalHeight: 0
    	};

    	let containerEl = null;
    	let containerRect = null;
    	let imgEl = null;
    	let dragStartPosition = { x: 0, y: 0 };
    	let dragStartCrop = { x: 0, y: 0 };
    	let lastPinchDistance = 0;
    	let rafDragTimeout = null;
    	let rafZoomTimeout = null;
    	let isDragging = false;
    	let dispatch = createEventDispatcher();

    	onMount(() => {
    		// when rendered via SSR, the image can already be loaded and its onLoad callback will never be called
    		if (imgEl && imgEl.complete) {
    			onImgLoad();
    		}
    	});

    	onDestroy(() => {
    		dispatch = null;
    	});

    	const onImgLoad = () => {
    		computeSizes();
    		emitCropData();
    	};

    	const getAspect = () => {
    		if (cropSize) {
    			return cropSize.width / cropSize.height;
    		}

    		return aspect;
    	};

    	const computeSizes = () => {
    		if (imgEl) {
    			imageSize = {
    				width: imgEl.width,
    				height: imgEl.height,
    				naturalWidth: imgEl.naturalWidth,
    				naturalHeight: imgEl.naturalHeight
    			};

    			$$invalidate(7, cropperSize = cropSize
    			? cropSize
    			: getCropSize(imgEl.width, imgEl.height, aspect));
    		}

    		if (containerEl) {
    			containerRect = containerEl.getBoundingClientRect();
    		}
    	};

    	const getMousePoint = e => ({
    		x: Number(e.clientX),
    		y: Number(e.clientY)
    	});

    	const getTouchPoint = touch => ({
    		x: Number(touch.clientX),
    		y: Number(touch.clientY)
    	});

    	const onMouseDown = e => {
    		isDragging = true;
    		onDragStart(getMousePoint(e));
    	};

    	const onMouseMove = e => {
    		if (!isDragging) {
    			return;
    		}

    		onDrag(getMousePoint(e));
    	};

    	const onTouchStart = e => {
    		if (e.touches.length === 2) {
    			onPinchStart(e);
    		} else if (e.touches.length === 1) {
    			onDragStart(getTouchPoint(e.touches[0]));
    		}
    	};

    	const onTouchMove = e => {
    		isDragging = true;

    		// Prevent whole page from scrolling on iOS.
    		if (e.touches.length === 2) {
    			onPinchMove(e);
    		} else if (e.touches.length === 1) {
    			onDrag(getTouchPoint(e.touches[0]));
    		}
    	};

    	const onDragStart = ({ x, y }) => {
    		dragStartPosition = { x, y };
    		dragStartCrop = { x: crop.x, y: crop.y };
    	};

    	const onDrag = ({ x, y }) => {
    		if (rafDragTimeout) window.cancelAnimationFrame(rafDragTimeout);

    		rafDragTimeout = window.requestAnimationFrame(() => {
    			if (x === undefined || y === undefined) return;
    			const offsetX = x - dragStartPosition.x;
    			const offsetY = y - dragStartPosition.y;

    			const requestedPosition = {
    				x: dragStartCrop.x + offsetX,
    				y: dragStartCrop.y + offsetY
    			};

    			$$invalidate(1, crop = restrictPosition$1
    			? restrictPosition(requestedPosition, imageSize, cropperSize, zoom)
    			: requestedPosition);
    		});
    	};

    	const onDragStopped = () => {
    		isDragging = false;
    		emitCropData();
    	};

    	const onPinchStart = e => {
    		const pointA = getTouchPoint(e.touches[0]);
    		const pointB = getTouchPoint(e.touches[1]);
    		lastPinchDistance = getDistanceBetweenPoints(pointA, pointB);
    		onDragStart(getCenter(pointA, pointB));
    	};

    	const onPinchMove = e => {
    		const pointA = getTouchPoint(e.touches[0]);
    		const pointB = getTouchPoint(e.touches[1]);
    		const center = getCenter(pointA, pointB);
    		onDrag(center);
    		if (rafZoomTimeout) window.cancelAnimationFrame(rafZoomTimeout);

    		rafZoomTimeout = window.requestAnimationFrame(() => {
    			const distance = getDistanceBetweenPoints(pointA, pointB);
    			const newZoom = zoom * (distance / lastPinchDistance);
    			setNewZoom(newZoom, center);
    			lastPinchDistance = distance;
    		});
    	};

    	const onWheel = e => {
    		const point = getMousePoint(e);
    		const newZoom = zoom - e.deltaY * zoomSpeed / 200;
    		setNewZoom(newZoom, point);
    	};

    	const getPointOnContainer = ({ x, y }) => {
    		if (!containerRect) {
    			throw new Error("The Cropper is not mounted");
    		}

    		return {
    			x: containerRect.width / 2 - (x - containerRect.left),
    			y: containerRect.height / 2 - (y - containerRect.top)
    		};
    	};

    	const getPointOnImage = ({ x, y }) => ({
    		x: (x + crop.x) / zoom,
    		y: (y + crop.y) / zoom
    	});

    	const setNewZoom = (newZoom, point) => {
    		const zoomPoint = getPointOnContainer(point);
    		const zoomTarget = getPointOnImage(zoomPoint);
    		$$invalidate(0, zoom = Math.min(maxZoom, Math.max(newZoom, minZoom)));

    		const requestedPosition = {
    			x: zoomTarget.x * zoom - zoomPoint.x,
    			y: zoomTarget.y * zoom - zoomPoint.y
    		};

    		$$invalidate(1, crop = restrictPosition$1
    		? restrictPosition(requestedPosition, imageSize, cropperSize, zoom)
    		: requestedPosition);
    	};

    	const emitCropData = () => {
    		if (!cropperSize || cropperSize.width === 0) return;

    		// this is to ensure the crop is correctly restricted after a zoom back (https://github.com/ricardo-ch/svelte-easy-crop/issues/6)
    		const position = restrictPosition$1
    		? restrictPosition(crop, imageSize, cropperSize, zoom)
    		: crop;

    		const { croppedAreaPercentages, croppedAreaPixels } = computeCroppedArea(position, imageSize, cropperSize, getAspect(), zoom, restrictPosition$1);

    		dispatch("cropcomplete", {
    			percent: croppedAreaPercentages,
    			pixels: croppedAreaPixels
    		});
    	};

    	function gesturestart_handler(event) {
    		bubble.call(this, $$self, event);
    	}

    	function gesturechange_handler(event) {
    		bubble.call(this, $$self, event);
    	}

    	function img_binding($$value) {
    		binding_callbacks[$$value ? "unshift" : "push"](() => {
    			imgEl = $$value;
    			$$invalidate(6, imgEl);
    		});
    	}

    	function div_binding($$value) {
    		binding_callbacks[$$value ? "unshift" : "push"](() => {
    			containerEl = $$value;
    			$$invalidate(8, containerEl);
    		});
    	}

    	$$self.$$set = $$props => {
    		if ("image" in $$props) $$invalidate(2, image = $$props.image);
    		if ("crop" in $$props) $$invalidate(1, crop = $$props.crop);
    		if ("zoom" in $$props) $$invalidate(0, zoom = $$props.zoom);
    		if ("aspect" in $$props) $$invalidate(17, aspect = $$props.aspect);
    		if ("minZoom" in $$props) $$invalidate(18, minZoom = $$props.minZoom);
    		if ("maxZoom" in $$props) $$invalidate(19, maxZoom = $$props.maxZoom);
    		if ("cropSize" in $$props) $$invalidate(20, cropSize = $$props.cropSize);
    		if ("cropShape" in $$props) $$invalidate(3, cropShape = $$props.cropShape);
    		if ("showGrid" in $$props) $$invalidate(4, showGrid = $$props.showGrid);
    		if ("zoomSpeed" in $$props) $$invalidate(21, zoomSpeed = $$props.zoomSpeed);
    		if ("crossOrigin" in $$props) $$invalidate(5, crossOrigin = $$props.crossOrigin);
    		if ("restrictPosition" in $$props) $$invalidate(22, restrictPosition$1 = $$props.restrictPosition);
    	};

    	$$self.$$.update = () => {
    		if ($$self.$$.dirty[0] & /*imgEl, cropSize, aspect*/ 1179712) {
    			// ------ Reactive statement ------
    			//when aspect changes, we reset the cropperSize
    			if (imgEl) {
    				$$invalidate(7, cropperSize = cropSize
    				? cropSize
    				: getCropSize(imgEl.width, imgEl.height, aspect));
    			}
    		}

    		if ($$self.$$.dirty[0] & /*zoom*/ 1) {
    			// when zoom changes, we recompute the cropped area
    			zoom && emitCropData();
    		}
    	};

    	return [
    		zoom,
    		crop,
    		image,
    		cropShape,
    		showGrid,
    		crossOrigin,
    		imgEl,
    		cropperSize,
    		containerEl,
    		onImgLoad,
    		computeSizes,
    		onMouseDown,
    		onMouseMove,
    		onTouchStart,
    		onTouchMove,
    		onDragStopped,
    		onWheel,
    		aspect,
    		minZoom,
    		maxZoom,
    		cropSize,
    		zoomSpeed,
    		restrictPosition$1,
    		gesturestart_handler,
    		gesturechange_handler,
    		img_binding,
    		div_binding
    	];
    }

    class Src extends SvelteComponent {
    	constructor(options) {
    		super();
    		if (!document.getElementById("svelte-12kodkg-style")) add_css();

    		init(
    			this,
    			options,
    			instance,
    			create_fragment,
    			safe_not_equal,
    			{
    				image: 2,
    				crop: 1,
    				zoom: 0,
    				aspect: 17,
    				minZoom: 18,
    				maxZoom: 19,
    				cropSize: 20,
    				cropShape: 3,
    				showGrid: 4,
    				zoomSpeed: 21,
    				crossOrigin: 5,
    				restrictPosition: 22
    			},
    			[-1, -1]
    		);
    	}
    }

    return Src;

})));
